/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.identity.auth;

/**
 * @author "Pierre Vigneras"
 * @date Dec 14, 2007
 */
public interface AuthenticationServiceOp {
  /**
   * Authenticate a user given some authentication strings.
   * 
   * The strings params can represent a classical pair login/password where
   * login can be a Unix login name, an email address, an OpenId, an LDAP cn,
   * ... depending on the actual implementation.
   * 
   * Usually the implementation of this interface comes with a corresponding
   * {@link LoginModule}. This LoginModule knows what informations are required
   * for this service, ask them to the user (using {@link Callback}) and forward
   * them to this service implementation.
   * 
   * On success, this implementation returns an id String that identifies the
   * authenticated user in the {@link IdentityServiceOp} implementation.
   * 
   * 
   * @param params
   *          an array of authentication strings (e.g.: login/password)
   * @return the id mapped to a UserOp in the related {@link IdentityServiceOp}
   *         implementation or null if the authentication failed.
   */
  String authenticateUser(String... params);
}
