/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.identity.auth;

import java.util.Map;

import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

/**
 * This {@link LoginModule} is used only in the case of an heavy Bonita client
 * (standalone, only one JVM, no J2EE). Its purpose is to store data into the
 * SecurityContext. It should be the last LoginModule in the stack of
 * LoginModule configuration.
 * 
 * @author "Pierre Vigneras"
 * @date Dec 5, 2007
 */
public final class StorageLoginModule implements LoginModule {
  // /**
  // * Property key to define the multi-user mode. Defined to be "multiuser".
  // * Property Value can be either "true" or "false". If set to "true",
  // * then the authenticated subject is registered (on commit() success)
  // * into the caller thread local {@link SecurityContext}. If set to "false",
  // * the authenticated subject is registered into a global
  // * {@link SecurityContext}.
  // *
  // */
  // public static final String MULTIUSER_OPTION_NAME = "multiuser";
  //
  protected Subject subject;

  public boolean abort() throws LoginException {
    return true;
  }

  public boolean commit() throws LoginException {
    SecurityContext.setSubject(subject);
    return true;
  }

  public void initialize(Subject subject, CallbackHandler arg1,
      Map<String, ? > arg2, Map<String, ? > arg3) {

    this.subject = subject;
  }

  public boolean login() throws LoginException {
    // final String multiuserMode = (String) options.get(MULTIUSER_OPTION_NAME);
    // if (multiuserMode == null) throw new
    // MissingOptionException(MULTIUSER_OPTION_NAME);
    // multiuser = Boolean.parseBoolean(multiuserMode);
    //
    return true;
  }

  public boolean logout() throws LoginException {
    SecurityContext.clearSubject();
    return true;
  }
}
